@@ -395,6 +395,7 @@ setClass("kpca", representation(pcv = "matrix",
                                 rotated = "matrix",
                                 xmatrix = "matrix",
                                 kcall = "ANY",
+                                kterms = "ANY",
                                 n.action = "ANY"))
 #accessor functions 
 if(!isGeneric("pcv")){
@@ -456,6 +457,12 @@ setReplaceMethod("kcall","kpca", function(x, value){
   x
 })
 
+setMethod("kterms","kpca", function(object) object@kterms)
+setReplaceMethod("kterms","kpca", function(x, value){
+  x@kterms <- value
+  x
+})
+
 setMethod("n.action","kpca", function(object) object@n.action)
 setReplaceMethod("n.action","kpca", function(x, value){
   x@n.action <- value
